<?php

use CRM_Bij1algemeen_ExtensionUtil as E;

/**
 * Form voor BIJ1 specifieke instellingen
 *
 * @author Erik Hommel (CiviCooP) <erik.hommel@civicoop.org>
 * @date 4 Jan 2022
 * @license AGPL-3.0
 * @see https://docs.civicrm.org/dev/en/latest/framework/quickform/
 */
class CRM_Bij1algemeen_Form_Instellingen extends CRM_Core_Form {

  /**
   * Overridden parent method to build form
   */
  public function buildQuickForm() {
    if ($this->_action == CRM_Core_Action::UPDATE) {
      CRM_Utils_System::setTitle(E::ts('BIJ1 Specifieke instellingen'));
      $this->add('text', 'bij1_basisbedrag', E::ts("Basis bedrag voor standaard lidmaatschap"), [], TRUE);
      $this->add('text', 'bij1_sociaaltariefbedrag', E::ts("Bedrag voor sociaal tarief lidmaatschap"), [], TRUE);
      $this->add('text', 'bij1_betalinggemist', E::ts("Aantal keren betaling gemist voor actie"), [], TRUE);
      $this->addRule('bij1_basisbedrag', E::ts('Moet een bedrag zijn'), 'money');
      $this->addRule('bij1_sociaaltariefbedrag', E::ts('Moet een bedrag zijn'), 'money');
      $this->addRule('bij1_betalinggemist', E::ts('Moet een geheel getal zijn'), 'positiveInteger');
      $this->addButtons([
        [
          'type' => 'next',
          'name' => E::ts('Save'),
          'isDefault' => TRUE,
        ],
        [
          'type' => 'cancel',
          'name' => E::ts('Cancel')
        ],
      ]);
      // export form elements
      $this->assign('elementNames', $this->getRenderableElementNames());
    }
    parent::buildQuickForm();
  }

  /**
   * Overridden parent method to process submitted form
   */
  public function postProcess() {
    if ($this->_action == CRM_Core_Action::UPDATE) {
      $values = $this->exportValues();
      if (isset($values['bij1_basisbedrag'])) {
        Civi::settings()->set('bij1_basisbedrag', $values['bij1_basisbedrag']);
      }
      if (isset($values['bij1_sociaaltariefbedrag'])) {
        Civi::settings()->set('bij1_sociaaltariefbedrag', $values['bij1_sociaaltariefbedrag']);
      }
      if (isset($values['bij1_betalinggemist'])) {
        Civi::settings()->set('bij1_betalinggemist', $values['bij1_betalinggemist']);
      }
    }
    CRM_Core_Session::setStatus(E::ts("BIJ1 Specifieke instellingen opgeslagen"), E::ts("Saved"), "success");
    parent::postProcess();
  }

  /**
   * Overridden parent method to set defaults
   *
   * @return array|NULL
   * @throws API_Exception
   */
  public function setDefaultValues() {
    $defaults = [];
    if ($this->_action == CRM_Core_Action::UPDATE) {
      $defaults['bij1_basisbedrag'] = Civi::settings()->get('bij1_basisbedrag');
      $defaults['bij1_sociaaltariefbedrag'] = Civi::settings()->get('bij1_sociaaltariefbedrag');
      $defaults['bij1_betalinggemist'] = Civi::settings()->get('bij1_betalinggemist');
    }
    return $defaults;
  }

  /**
   * Get the fields/elements defined in this form.
   *
   * @return array (string)
   */
  public function getRenderableElementNames() {
    // The _elements list includes some items which should not be
    // auto-rendered in the loop -- such as "qfKey" and "buttons".  These
    // items don't have labels.  We'll identify renderable by filtering on
    // the 'label'.
    $elementNames = array();
    foreach ($this->_elements as $element) {
      /** @var HTML_QuickForm_Element $element */
      $label = $element->getLabel();
      if (!empty($label)) {
        $elementNames[] = $element->getName();
      }
    }
    return $elementNames;
  }

}
