<?php
use CRM_Bij1migratie_ExtensionUtil as E;

/**
 * Keuzewaarden (option values) aanmaken als ze nog niet bestaan
 *
 * @author Erik Hommel(CiviCooP) <erik.hommel@civicoop.org>
 * @license AGPL-3.0
 */
class CRM_Bij1algemeen_KeuzeWaarde {

  /**
   * Method om option value aan te maken
   * @param array $waardeData
   */
  public function maak(array $waardeData) {
    if (!empty($waardeData) && isset($waardeData['option_group_id:name']) && !empty($waardeData['option_group_id:name'])) {
      if (!isset($waardeData['label'])) {
        $delen = explode("_", $waardeData['name']);
        foreach ($delen as $deelKey => $deel) {
          $delen[$deelKey] = ucfirst(strtolower($deel));
        }
        $waardeData['label'] = implode(" ", $delen);
      }
      if (!$this->bestaat($waardeData['option_group_id:name'], $waardeData['name'])) {
        try {
          $optionValue = \Civi\Api4\OptionValue::create()
            ->addValue('option_group_id:name', $waardeData['option_group_id:name'])
            ->addValue('name', $waardeData['name'])
            ->addValue('label', $waardeData['label'])
            ->addValue('is_active', TRUE)
            ->addValue('is_reserved', TRUE);
          if (isset($waardeData['description']) && !empty($waardeData['description'])) {
            $optionValue->addValue('description', $waardeData['description']);
          }
          $optionValue->execute();
        }
        catch (API_Exception $ex) {
          Civi::log()->error("Kon geen keuzewaarde aanmaken met data: " . json_encode($waardeData) . ", foutboodschap van API4 OptionValue create: " . $ex->getMessage());
        }
      }
    }
  }

  /**
   * Method om te checken of de keuzewaarde al bestaat
   *
   * @param string $optionGroupNaam
   * @param string $optionValueNaam
   * @return bool
   */
  public function bestaat(string $optionGroupNaam, string $optionValueNaam) {
    if (!empty($optionGroupNaam) && !empty($optionValueNaam)) {
      try {
        $optionValues = \Civi\Api4\OptionValue::get()
          ->addSelect('COUNT(*) AS count')
          ->addWhere('option_group_id:name', '=', $optionGroupNaam)
          ->addWhere('name', '=', $optionValueNaam)
          ->execute();
        foreach ($optionValues as $optionValue) {
          if ($optionValue['count'] > 0) {
            return TRUE;
          }
        }
      }
      catch (API_Exception $ex) {
        Civi::log()->error("Kon geen keuzewaarde zoeken met option group naam " . $optionGroupNaam . " en option value naam"
          . $optionValueNaam . " in " . __METHOD__ . ", foutboodschap van API4 OptionValue get: " . $ex->getMessage());
      }
    }
    return FALSE;
  }

  /**
   * Method om lidmaatschapsaanvraag activity type id op te halen
   * (eenmalig omdat tijdens de install de service het type nog niet kent)
   *
   * @return false|string
   */
  public function haalLidmaatschapsaanvraagTypeId() {
    $query = "SELECT cov.value
        FROM civicrm_option_group AS cog JOIN civicrm_option_value AS cov ON cog.id = cov.option_group_id
        WHERE cog.name = %1 AND cov.name = %2 LIMIT 1";
    $activityTypeId = \CRM_Core_DAO::singleValueQuery($query, [
      1 => ["activity_type", "String"],
      2 => ["lidmaatschapsaanvraag", "String"],
    ]);
    if ($activityTypeId) {
      return $activityTypeId;
    }
    return FALSE;
  }

  /**
   * Method om de data voor lidmaatschapsaanvraag op te nemen
   *
   * @return array
   */
  public function getLidmaatschapsaanvraagData() {
    return [
      'option_group_id:name' => 'activity_type',
      'name' => 'lidmaatschapsaanvraag',
      'label' => 'Lidmaatschapsaanvraag',
      'description' => "Activiteitstype voor een aanvraag van lidmaatschap met specifieke gegevens van de aanvraag"
    ];
  }

  /**
   * Method om de data voor Mollie betaalmethodes op te nemen
   *
   * @return array
   */
  public function getMollieBetaalMethodes() {
    return [
      [
        'option_group_id:name' => 'payment_instrument',
        'name' => 'mollie_frst',
        'label' => 'Mollie First',
        'description' => "Betaalmethode voor Mollie First transactie",
        'weight' => '90',
      ],
      [
        'option_group_id:name' => 'payment_instrument',
        'name' => 'mollie_rcur',
        'label' => 'Mollie Recurring',
        'description' => "Betaalmethode voor Mollie Recurring transactie",
        'weight' => '100',
      ],
      [
        'option_group_id:name' => 'payment_instrument',
        'name' => 'mollie_ooff',
        'label' => 'Mollie One-off',
        'description' => "Betaalmethode voor Mollie One-off transactie",
        'weight' => '110',
      ],
    ];
  }

}
