<?php
use CRM_Bij1algemeen_ExtensionUtil as E;

/**
 * Collection of upgrade steps.
 */
class CRM_Bij1algemeen_Upgrader extends CRM_Bij1algemeen_Upgrader_Base {

  // By convention, functions that look like "function upgrade_NNNN()" are
  // upgrade tasks. They are executed in order (like Drupal's hook_update_N).

  /**
   * Eigen velden en groepen voor Bij1 genereren bij install
   */
  public function install() {
    $keuzeWaarde = new CRM_Bij1algemeen_KeuzeWaarde();
    $keuzeWaarde->maak($keuzeWaarde->getLidmaatschapsaanvraagData());
    $betaalMethodes = $keuzeWaarde->getMollieBetaalMethodes();
    foreach ($betaalMethodes as $betaalMethode) {
      $keuzeWaarde->maak($betaalMethode);
    }
    $customData = new CRM_Bij1algemeen_CustomData();
    $customData->maak($customData->getLidData());
    $customData->maak($customData->getAirtableData());
    $customData->maak($customData->getMollieBetalingData());
    $customData->maak($customData->getLidmaatschapsaanvraagData());
    $groep = new CRM_Bij1algemeen_Groep();
    $groep->maakLedenGroepen();
    $identity = new CRM_Bij1algemeen_IdentityTracker();
    $identity->setIsReserved();
    $identity->maakMollieCustomerIdType();
    Civi::settings()->set('bij1_basisbedrag', 30);
    Civi::settings()->set('bij1_sociaaltariefbedrag', 15);
    Civi::settings()->set('bij1_betalinggemist', 3);
    Civi::settings()->set('bij1_laatste_lidnummer', 1);
  }

  /**
   * Upgrade 1000: add airtable custom group
   *
   * @return TRUE on success
   * @throws Exception
   */
  public function upgrade_1000() {
    $this->ctx->log->info('Applying update 1000 - add airtable data');
    $customData = new CRM_Bij1algemeen_CustomData();
    $customData->maak($customData->getAirtableData());
    return TRUE;
  }

  /**
   * Upgrade 1010: remove mollie customer id table -> is identity
   *
   * @return TRUE on success
   * @throws Exception
   */
  public function upgrade_1010() {
    $this->ctx->log->info('Applying update 1010 - remove mollie customer id table -> is identity');
    try {
      \Civi\Api4\CustomGroup::delete()
        ->addWhere('name', '=', 'mollie_customer_ids')
        ->execute();
    }
    catch (API_Exception $ex) {
    }
    return TRUE;
  }

  /**
   * Upgrade 1020: add Curacao as country
   *
   * @return TRUE on success
   * @throws Exception
   */
  public function upgrade_1020() {
    $this->ctx->log->info('Applying update 1020 - add curacao');
    $count = CRM_Core_DAO::singleValueQuery("SELECT COUNT(*) FROM civicrm_country WHERE iso_code = %1", [1 => ["CW", "String"]]);
    if ($count == 0) {
      $insert = "INSERT INTO civicrm_country (name, iso_code, region_id) VALUES(%1, %2, %3)";
      CRM_Core_DAO::executeQuery($insert, [
        1 => ["Curaçao", "String"],
        2 => ["CW", "String"],
        3 => [2, "Integer"],
     ]);
    }
    return TRUE;
  }

  /**
   * Upgrade 1030: add mollie betaling data
   *
   * @return TRUE on success
   * @throws Exception
   */
  public function upgrade_1030() {
    $this->ctx->log->info('Applying update 1030 - add mollie betaling data');
    $customData = new CRM_Bij1algemeen_CustomData();
    $customData->maak($customData->getMollieBetalingData());
    return TRUE;
  }

  /**
   * Upgrade 1040: add lidmaatschapsaanvraag activity type en custom data
   *
   * @return TRUE on success
   * @throws Exception
   */
  public function upgrade_1040() {
    $this->ctx->log->info('Applying update 1040 - add lidmaatschapsaanvraag activity type en custom data');
    $keuzeWaarde = new CRM_Bij1algemeen_KeuzeWaarde();
    $keuzeWaarde->maak($keuzeWaarde->getLidmaatschapsaanvraagData());
    $customData = new CRM_Bij1algemeen_CustomData();
    $customData->maak($customData->getLidmaatschapsaanvraagData());
    return TRUE;
  }

  /**
   * Upgrade 1050: bij1 specifieke instellingen
   *
   * @return TRUE on success
   * @throws Exception
   */
  public function upgrade_1050() {
    $this->ctx->log->info('Applying update 1050 - bij1 specifieke instellingen');
    Civi::settings()->set('bij1_basisbedrag', 30);
    Civi::settings()->set('bij1_sociaaltariefbedrag', 15);
    Civi::settings()->set('bij1_betalinggemist', 3);
    return TRUE;
  }

  /**
   * Upgrade 1060: mollie betaalmethodes toevoegen
   *
   * @return TRUE on success
   * @throws Exception
   */
  public function upgrade_1060() {
    $this->ctx->log->info('Applying update 1060 - mollie betaal methodes');
    $keuzeWaarde = new CRM_Bij1algemeen_KeuzeWaarde();
    $betaalMethodes = $keuzeWaarde->getMollieBetaalMethodes();
    foreach ($betaalMethodes as $betaalMethode) {
      $keuzeWaarde->maak($betaalMethode);
    }
    return TRUE;
  }

  /**
   * Upgrade 1070: lidnummer custom veld (en groep) toevoegen + hernoem lidnummer airtable custom veld
   *
   * @return TRUE on success
   * @throws Exception
   */
  public function upgrade_1070() {
    $this->ctx->log->info('Applying update 1070 - lidnummer custom veld (en groep) toevoegen');
    $customData = new CRM_Bij1algemeen_CustomData();
    $customData->maak($customData->getLidData());
    if (CRM_Core_BAO_SchemaHandler::checkIfFieldExists('civicrm_value_airtable_data', 'lidnummer')) {
      CRM_Core_DAO::executeQuery("ALTER TABLE `civicrm_value_airtable_data` CHANGE `lidnummer` `lidnummer_airtable` VARCHAR(32)");
    }
    return TRUE;
  }

  /**
   * Upgrade 1080: introduceer setting voor laatst uitgegeven lidnummer
   *
   * @return TRUE on success
   * @throws Exception
   */
  public function upgrade_1080() {
    $this->ctx->log->info('Applying update 1080 - introduceer setting voor laatst uitgegeven lidnummer');
    Civi::settings()->set('bij1_laatste_lidnummer', 0);
    return TRUE;
  }

  /**
   * Example: Work with entities usually not available during the install step.
   *
   * This method can be used for any post-install tasks. For example, if a step
   * of your installation depends on accessing an entity that is itself
   * created during the installation (e.g., a setting or a managed entity), do
   * so here to avoid order of operation problems.
   */
  // public function postInstall() {
  //  $customFieldId = civicrm_api3('CustomField', 'getvalue', array(
  //    'return' => array("id"),
  //    'name' => "customFieldCreatedViaManagedHook",
  //  ));
  //  civicrm_api3('Setting', 'create', array(
  //    'myWeirdFieldSetting' => array('id' => $customFieldId, 'weirdness' => 1),
  //  ));
  // }

  /**
   * Example: Run an external SQL script when the module is uninstalled.
   */
  // public function uninstall() {
  //  $this->executeSqlFile('sql/myuninstall.sql');
  // }

  /**
   * Example: Run a simple query when a module is enabled.
   */
  // public function enable() {
  //  CRM_Core_DAO::executeQuery('UPDATE foo SET is_active = 1 WHERE bar = "whiz"');
  // }

  /**
   * Example: Run a simple query when a module is disabled.
   */
  // public function disable() {
  //   CRM_Core_DAO::executeQuery('UPDATE foo SET is_active = 0 WHERE bar = "whiz"');
  // }

  /**
   * Example: Run an external SQL script.
   *
   * @return TRUE on success
   * @throws Exception
   */
  // public function upgrade_4201() {
  //   $this->ctx->log->info('Applying update 4201');
  //   // this path is relative to the extension base dir
  //   $this->executeSqlFile('sql/upgrade_4201.sql');
  //   return TRUE;
  // }


  /**
   * Example: Run a slow upgrade process by breaking it up into smaller chunk.
   *
   * @return TRUE on success
   * @throws Exception
   */
  // public function upgrade_4202() {
  //   $this->ctx->log->info('Planning update 4202'); // PEAR Log interface

  //   $this->addTask(E::ts('Process first step'), 'processPart1', $arg1, $arg2);
  //   $this->addTask(E::ts('Process second step'), 'processPart2', $arg3, $arg4);
  //   $this->addTask(E::ts('Process second step'), 'processPart3', $arg5);
  //   return TRUE;
  // }
  // public function processPart1($arg1, $arg2) { sleep(10); return TRUE; }
  // public function processPart2($arg3, $arg4) { sleep(10); return TRUE; }
  // public function processPart3($arg5) { sleep(10); return TRUE; }

  /**
   * Example: Run an upgrade with a query that touches many (potentially
   * millions) of records by breaking it up into smaller chunks.
   *
   * @return TRUE on success
   * @throws Exception
   */
  // public function upgrade_4203() {
  //   $this->ctx->log->info('Planning update 4203'); // PEAR Log interface

  //   $minId = CRM_Core_DAO::singleValueQuery('SELECT coalesce(min(id),0) FROM civicrm_contribution');
  //   $maxId = CRM_Core_DAO::singleValueQuery('SELECT coalesce(max(id),0) FROM civicrm_contribution');
  //   for ($startId = $minId; $startId <= $maxId; $startId += self::BATCH_SIZE) {
  //     $endId = $startId + self::BATCH_SIZE - 1;
  //     $title = E::ts('Upgrade Batch (%1 => %2)', array(
  //       1 => $startId,
  //       2 => $endId,
  //     ));
  //     $sql = '
  //       UPDATE civicrm_contribution SET foobar = whiz(wonky()+wanker)
  //       WHERE id BETWEEN %1 and %2
  //     ';
  //     $params = array(
  //       1 => array($startId, 'Integer'),
  //       2 => array($endId, 'Integer'),
  //     );
  //     $this->addTask($title, 'executeSql', $sql, $params);
  //   }
  //   return TRUE;
  // }

}
